import pandas as pd


class Candidate:
    def __init__(self, info) -> None:
        self.party = info["party"]
        self.name = info["name"]
        self.votes = info["votes"]
        self.share = info["share"]


class Party:
    PARTY_COLOURS = {
        "Con": "#0087DC",
        "Lab": "#E4003B",
        "LD": "#FAA61A",
        "SNP": "#FDF38E",
        "DUP": "#D46A4C",
        "Green": "#02A95B",
        "SF": "#326760",
        "SDLP": "#2AA82C",
        "Ref": "#12B6CF",
        "PC": "#005B54",
        "APNI": "#F6CB2F",
        "Spk": "black"
    }
    def __init__(self, name) -> None:
        self.name = name[0]
        self.abbrv = name[1]
        self.seats = 0
        self.seat_list = []
        self.colour = Party.PARTY_COLOURS.get(self.abbrv, "grey")


class Parties(dict):
    def get(self, key: tuple[str, str] | str) -> Party:
        if key[0] == "Labour and Co-operative":
           key = ("Labour", key[1])
        if isinstance(key, tuple):
            if not key in self:
                self[key] = Party(key)
            return self[key]
        # if key in [x[0] for x in self.keys()] + [x[1] for x in self.keys()]:
        for k, v in self.items():
            if key in k:
                return v
        raise ValueError("Parties.get argument must be a tuple")


class Constituency:
    def __init__(self, election, name) -> None:
        self.name = name
        self.candidates = []
        self.election = election

    def add_data(self, row) -> None:
        self.candidates.append(Candidate({
            "party": self.election.parties.get((row["Party name"], row["Party abbreviation"])),
            "name": row["Candidate first name"] + " " + row["Candidate surname"],
            "votes": int(row["Votes"]),
            "share": float(row["Share"])
        }))
        self.candidates.sort(key=lambda d: d.votes, reverse=True)
        self.winner = self.candidates[0]


class Election:
    def __init__(self, filename) -> None:
        try:
            df = pd.read_csv(filename)
        except FileNotFoundError as e:
            raise ValueError(e.args) # value is so that argparse catches it without me having to do fancy stuff : https://docs.python.org/3/library/argparse.html#type
        self.constituencies = {}
        self.parties = Parties()
        for index, row in df.iterrows():
            name = row["Constituency name"]
            if not name in self.constituencies.keys():
                self.constituencies[name] = Constituency(self, name)
            self.constituencies[name].add_data(row)

        for c in self.constituencies.values():
            c.winner.party.seat_list.append(c)
            c.winner.party.seats += 1
        self.parties = Parties(
            sorted(self.parties.items(), key=lambda i: i[1].seats, reverse=True))

def CSVElectionType(filename):
    with open(filename) as file:
        return Election(filename)



from argparse import ArgumentParser
from . import heatmap
from . import bar

runners = {
    "heatmap": heatmap.run,
    "bar": bar.run
}

def main():
    parser = ArgumentParser(prog="election-stats")
    parser.add_argument("-d", "--debug", action="store_true", default=False, help="enable debug mode")
    subparsers = parser.add_subparsers(dest="command", required=True, title="commands", help="%(help)s")
    heatmap_parser = subparsers.add_parser("heatmap", help="create heatmaps comparing two election results")
    heatmap.parse(heatmap_parser)
    bar_parser = subparsers.add_parser("bar", help="create bar charts showing simple results")
    bar.parse(bar_parser)
    args = parser.parse_args()
    runners[args.command](args)


from .election import Election
import seaborn as sns
import matplotlib.pyplot as plt
import pandas as pd

def parse(parser):
    parser.add_argument("election", type=Election)

def run(args):
    print("Bar chart time!")
    
    election = args.election
    print(election.parties)
    print([(p.name,p.seats) for p in election.parties.values()])
    data = {p.name:p.seats for p in election.parties.values()}
    data = dict(filter(lambda i:i[1], data.items()))
    print(data)
    df = pd.DataFrame(data, index=[0])
    print(df)
    c = [election.parties.get(x).colour for x in df]
    sns.barplot(df, palette=c)
    plt.show()


from .election import Election
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

def parse(parser):
    parser.add_argument("primary", help="Primary data file", type=Election)
    parser.add_argument("secondary", help="Secondary data file", type=Election)

def run(args):
    print(args)
    e1 = args.primary
    e2 = args.secondary
    heatmap_data = {}
    # for name in e2.constituencies.keys():
    #     if not e2.constituencies[name].candidates[0][dn] in heatmap_data.keys():
    #         heatmap_data[e2.constituencies[name].candidates[0][dn]] = {}
    #     try:
    #         heatmap_data[e2.constituencies[name].candidates[0][dn]
    #                      ][e1.constituencies[name].candidates[0][dn]] += 1
    #     except KeyError:
    #         heatmap_data[e2.constituencies[name].candidates[0][dn]
    #                      ][e1.constituencies[name].candidates[0][dn]] = 1
    
    ## Whole algorithm could be improved...
    ## those try... except clauses are only at runtime
    for c in e2.constituencies.values():
        try:
            heatmap_data[c.winner.party.abbrv][e1.constituencies[c.name]] += 1
        except:
            try:
                heatmap_data[c.winner.party.abbrv][e1.constituencies[c.name].winner.party.abbrv] = 1
            except:
                heatmap_data[c.winner.party.abbrv] = {}
                heatmap_data[c.winner.party.abbrv][e1.constituencies[c.name].winner.party.abbrv] = 1
    
    
    heatmap_data["Total"] = {name: sum(
        [d.get(name, 0) for d in heatmap_data.values()]) for name in heatmap_data.keys()}
    heatmap_data = {k: (v | {"Total":sum(heatmap_data[k].values())}) for k, v in heatmap_data.items()}
    del heatmap_data["Total"]["Total"]
    print(heatmap_data)

    heatmapdf = pd.DataFrame(heatmap_data)
    srt = list(e1.parties.keys())
    heatmapdf = heatmapdf.reindex(srt + ["Total"], axis=0)
    heatmapdf = heatmapdf.reindex(["Total"] + srt, axis=1)
    print(heatmapdf)
    # fmt prevents scientific notation. yes, it is a float with 0 prisicion - this is to support NaN
    ax = sns.heatmap(heatmapdf, cmap="Blues", annot=True,
                     fmt=".0f", linewidths=.5)
    ax.set_title("Change of Seats from 2017-2019 UK General Elections")
    ax.set_xlabel("2017 Seats")
    ax.set_ylabel("2019 Seats")
    plt.show()

